-- Battery_cmd.lua
-- Battery Control Routines

--====================================================================
-- this script controls the battery
-- implementing the clickable buttons which make various components visible
--====================================================================
-- ----------------------------
-- define tables of visibility states
-- ----------------------------

-- table of all affected objects
-- this is used to hide them all before drawing the currently visible ones

bt_all = {
"ZeusBattery",
"Charge00",
"Charge01",
"Charge02",
"Charge03",
"Charge04",
"Charge05",
"Charge06",
"Charge07",
"Charge08",
"Charge09",
"Charge10",

"BatteryI",
"Charge00I",
"Charge01I",
"Charge02I",
"Charge03I",
"Charge04I",
"Charge05I",
"Charge06I",
"Charge07I",
"Charge08I",
"Charge09I",
"Charge10I",

"BatteryControlButton",

"BatteryInst",
"BatteryChargeButton",
"BatteryDischargeButton",
"BatteryCycleButtonLit",
"BatteryStopButtonLit",
"BatteryExitButton",
"BatteryCycleButtonDim",
"BatteryStopButtonDim",

"BatteryRun",
"BatteryRunButton",
}

bt_charge = {
[0] = "Charge00",
[1] = "Charge01",
[2] = "Charge02",
[3] = "Charge03",
[4] = "Charge04",
[5] = "Charge05",
[6] = "Charge06",
[7] = "Charge07",
[8] = "Charge08",
[9] = "Charge09",
[10] = "Charge10",
}

bt_allI = {
"BatteryI",
"Charge00I",
"Charge01I",
"Charge02I",
"Charge03I",
"Charge04I",
"Charge05I",
"Charge06I",
"Charge07I",
"Charge08I",
"Charge09I",
"Charge10I",
}
bt_chargeI = {
[0] = "Charge00I",
[1] = "Charge01I",
[2] = "Charge02I",
[3] = "Charge03I",
[4] = "Charge04I",
[5] = "Charge05I",
[6] = "Charge06I",
[7] = "Charge07I",
[8] = "Charge08I",
[9] = "Charge09I",
[10] = "Charge10I",
}

bt_maint = {
"BatteryInst",
"BatteryI",
"BatteryChargeButton",
"BatteryDischargeButton",
"BatteryCycleButtonLit",
"BatteryStopButtonDim",
"BatteryExitButton",
}

bt_battery = {
"ZeusBattery",
"BatteryControlButton",
}

bt_SoftwareFailure = {
"SoftwareFailure",
}

bt_run = {
"BatteryRun",
"BatteryRunButton",
"BatteryExplosion",
}

-- ----------------------------
-- functions to hide and show the objects
-- ----------------------------

-- ---------------
bc_show = function (name)
   celestia:find(name):setvisible(true)
end

-- ---------------
bc_hide = function (name)
   celestia:find(name):setvisible(false)
end
-- ---------------

bc_show_tbl = function (map_table)
  for j,r in ipairs(map_table) do
    bc_show(map_table[j])
  end -- for j,r loop
end
-- ---------------
bc_hide_tbl = function (map_table)
  for j,r in ipairs(map_table) do
   celestia:find(map_table[j]):setvisible(false)
  end -- for j,r loop
end

bc_show_map = function (map_table)
  bc_hide_tbl(bt_all)
  bc_show_tbl (map_table)
end
--====================================================================
-- command routines
--====================================================================
bv_j = 6

bv_eT0 = nil
bv_exp = nil
bv_eR0 = nil

-- BatteryChargeButton = {bc_cp1, nil},
bc_cp1 = function ()
  bc_stop()
  if (celestia:find("Charge00I"):visible() )
  then
    bc_hide("Charge00I")
    bc_show("Charge01I")
    return
  elseif ( celestia:find("Charge10I"):visible() )
  then

  -- start of explosion 
    bc_hide_tbl(bt_all)
    bc_show_tbl(bt_run)
    bv_eT0 = celestia:gettime()
    bv_exp = true
    local obj = celestia:find("BatteryExplosion")
    bv_eR0 = obj:radius()
    return

  end
  for j,r in ipairs(bt_chargeI) do
   if (celestia:find(bt_chargeI[j]):visible() )
   then
     bc_hide(bt_chargeI[j])
     bv_j = j+1
     bc_show(bt_chargeI[bv_j])
     return
   end
  end -- for j,r loop
end

-- BatteryDischargeButton = {bc_cm1, nil},
bc_cm1 = function ()

  bc_stop()
  if (celestia:find("Charge00I"):visible() )
  then
    bc_show_map (bt_SoftwareFailure)
    return
  end
  for j,r in ipairs(bt_chargeI) do
   if (celestia:find(bt_chargeI[j]):visible() )
   then
     bc_hide(bt_chargeI[j])
     bv_j = j-1
     bc_show(bt_chargeI[bv_j])
     return
   end
  end -- for j,r loop
end

-- BatteryExitButton = {bc_exit, nil},
bc_exit = function ()
  bc_stop()
  bc_show_map (bt_battery)

-- show the corresponding charge value in small image
  bc_show(bt_charge[bv_j])
end

-- Batterycontrol = {bc_control, map},
bc_control = function (map)
  bc_show_map (map)

-- show the corresponding charge value in large image
   bc_show(bt_chargeI[bv_j])
end

-- ========
-- cycle through battery charge values

b_cycle = nil
b_T0 = 0
b_delta = 1

-- BatteryCycleButtonLit = {bc_cycle, this_date},
bc_cycle = function ()
  b_cycle = true
  b_T0 = celestia:gettime()
  bc_show("BatteryCycleButtonDim")
  bc_hide("BatteryCycleButtonLit")
  bc_show("BatteryStopButtonLit")
  bc_hide("BatteryStopButtonDim")
  
end

-- BatteryStopButtonLit = {bc_stop, nil},
bc_stop = function ()
  b_cycle = nil
  b_T0 = 0
  bc_hide("BatteryCycleButtonDim")
  bc_show("BatteryCycleButtonLit")
  bc_hide("BatteryStopButtonLit")
  bc_show("BatteryStopButtonDim")
end

bc_timers = function()

-- increase radius of explosion
  if (bv_exp)
  then
    local bv_eTnow = celestia:gettime()
    local obj = celestia:find("BatteryExplosion")
    local bv_rad = bv_eR0 + 200000.0*(bv_eTnow - bv_eT0)
    obj:setradius(bv_rad)
    if (bv_rad >= 25.0)
    then
    bc_hide("BatteryExplosion")
    bc_show("BatteryHide")
    bv_exp = nil
    end

-- do battery charge cycles
  elseif (b_cycle)
  then
--    if current time > 1 sec + previous time
--    show next charge
    local date = celestia:gettime()
    local onesec = 1.0/86400.0 -- fraction of a day
    local delta = date - b_T0
    if (delta >= onesec )
    then
      b_T0 = date
      for j,r in ipairs(bt_chargeI) do
       if (celestia:find(bt_chargeI[j]):visible() )
       then
         bc_hide_tbl(bt_chargeI)
         bc_show(bt_chargeI[j+b_delta])
	 bv_j = j+b_delta
         if (j >= 8) then b_delta = -1 end
         if (j <= 2) then b_delta = 1 end
	 return
       end
     end  
   end
 end
end

-- run away!
bc_run = function ()
    local obs = celestia:getobserver()
    local obj = celestia:find("BatteryExplosion")
    celestia:select(obj)
    obs:track(obj)
    obs:gotodistance(obj,500.0,5.0)
end


--====================================================================
-- command processor table and routines:
-- the name of the selected object is interpreted as a command
--====================================================================

-- b_selP = previous selection
-- to be restored immediately after recognizing a command
-- assume initial selection (if any) when Celestia starts isn't a command

-- must be initialized
-- but can't use getselection during startup of Celestia
-- which is when this code is initially executed
-- because nothing has been selected yet

b_selP = nil

-- ======================================= 
-- table of commands:
-- command text (i.e.name of selected object) = { command function, arg_of_fn },

bt_cmds = {
 BatteryControlButton = {bc_control,bt_maint},
 BatteryChargeButton = {bc_cp1, nil},
 BatteryDischargeButton = {bc_cm1, nil},
 BatteryCycleButtonLit = {bc_cycle, nil},
 BatteryStopButtonLit = {bc_stop, nil},
 BatteryExitButton = {bc_exit, nil},
 BatteryRunButton = {bc_run, nil},
  }


-- ======================================= 
-- action function

bc_cmds = function (this_date)

   bc_timers (this_date)

-- to implement commands only for the battery and its children
-- this code uses locals, bc_ routines and st_ tables

  local sel = celestia:getselection()

  if (sel == b_selP) then return 0 end
  
  local sname = sel:name()
  
-- get the two-element table (if any) matching the name of the selection
  local todo = bt_cmds[sname]
  
    if todo then 
     -- restore previous selection before executing this one
        if (b_selP ~= nil) then celestia:select(b_selP) end
	
	-- call the function [table element 1] associated with this command,
	-- passing it a single argument [table element 2]
	todo[1](todo[2])
	return 1
    end

-- This selection is not recognized as a command,
-- so save it for reversion after next command

	b_selP = sel
	return 0
end
